@@ -39,7 +39,7 @@ def inject_dataloader_value_from_model_config(model_cfg: dict, dataloader_cfg: D
     """
     if key not in model_cfg:
         logging.info(
-            f"Model level config does not container `{key}`, please explicitly provide `{key}` to the dataloaders."
+            f"Model level config does not contain `{key}`, please explicitly provide `{key}` to the dataloaders."
         )
         return
 
@@ -162,6 +162,12 @@ def get_tarred_dataset(
     tarred_audio_filepaths = convert_to_config_list(tarred_audio_filepaths)
     manifest_filepaths = convert_to_config_list(manifest_filepaths)
 
+    bucketing_weights = config.get('bucketing_weights', None)  # For upsampling buckets
+    if bucketing_weights:
+        for idx, weight in enumerate(bucketing_weights):
+            if not isinstance(weight, int) or weight <= 0:
+                raise ValueError(f"bucket weights must be positive integers")
+
     if len(manifest_filepaths) != len(tarred_audio_filepaths):
         raise ValueError(
             f"manifest_filepaths (length={len(manifest_filepaths)}) and tarred_audio_filepaths (length={len(tarred_audio_filepaths)}) need to have the same number of buckets."
@@ -216,8 +222,10 @@ def get_tarred_dataset(
                 world_size=world_size,
                 return_sample_id=config.get('return_sample_id', False),
             )
-
-        datasets.append(dataset)
+        if bucketing_weights:
+            [datasets.append(dataset) for _ in range(bucketing_weights[dataset_idx])]
+        else:
+            datasets.append(dataset)
 
     return get_chain_dataset(datasets=datasets, ds_config=config)
 
@@ -404,17 +412,31 @@ def get_chain_dataset(datasets, ds_config):
 
 def calc_bucketing_batch_sizes(ds_config, datasets_len):
     bucketing_batch_size = ds_config['bucketing_batch_size']
+    bucketing_weights = ds_config.get('bucketing_weights', None)  # To adjust for upsampled buckets
+
+    bucketing_batch_sizes = []
+
     if ds_config['batch_size'] != 1:
         raise ValueError(
             f"batch_size should be set to one when bucketing_batch_size is set and adaptive bucketing is enabled (batch_size={ds_config['batch_size']}!"
         )
-    if type(bucketing_batch_size) == int:
-        bucketing_batch_sizes = []
-        for idx in range(datasets_len):
-            scale_factor = datasets_len - idx
-            bucketing_batch_sizes.append(scale_factor * bucketing_batch_size)
-    elif isinstance(bucketing_batch_size, ListConfig) or isinstance(bucketing_batch_size, list):
-        bucketing_batch_sizes = bucketing_batch_size
+    if type(bucketing_batch_size) == int:  # linear scaling
+        if bucketing_weights:  # Want same batchsize for the same duplicated bucket
+            for idx, weight in enumerate(bucketing_weights):
+                scale_factor = datasets_len - idx
+                [bucketing_batch_sizes.append(scale_factor * bucketing_batch_size) for _ in range(weight)]
+        else:
+            for idx in range(datasets_len):
+                scale_factor = datasets_len - idx
+                bucketing_batch_sizes.append(scale_factor * bucketing_batch_size)
+    elif isinstance(bucketing_batch_size, ListConfig) or isinstance(
+        bucketing_batch_size, list
+    ):  # assigned bucket sizes
+        if bucketing_weights:  # Want same batchsize for same duplicated bucket
+            for idx, weight in enumerate(bucketing_weights):
+                [bucketing_batch_sizes.append(bucketing_batch_size[idx]) for _ in range(weight)]
+        else:
+            bucketing_batch_sizes = bucketing_batch_size
     else:
         raise ValueError(
             f"bucketing_batch_size should be an integer or a list (bucketing_batch_size={bucketing_batch_size})!"
