@@ -12,18 +12,87 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from dataclasses import dataclass
 from typing import List
 
 import editdistance
 import torch
 from torchmetrics import Metric
 
-from nemo.collections.asr.metrics.wer import move_dimension_to_the_front
-from nemo.collections.asr.parts.utils.rnnt_utils import Hypothesis
+from nemo.collections.asr.metrics.wer import AbstractCTCDecoding, CTCDecodingConfig
 from nemo.collections.common.tokenizers.tokenizer_spec import TokenizerSpec
 from nemo.utils import logging
 
 
+class CTCBPEDecoding(AbstractCTCDecoding):
+    """
+    Used for performing CTC auto-regressive / non-auto-regressive decoding of the logprobs.
+
+    Args:
+        decoding_cfg: A dict-like object which contains the following key-value pairs.
+            strategy: str value which represents the type of decoding that can occur.
+                Possible values are :
+                -   greedy (for greedy decoding).
+
+            compute_timestamps: A bool flag, which determines whether to compute the character/subword, or
+                word based timestamp mapping the output log-probabilities to discrite intervals of timestamps.
+                The timestamps will be available in the returned Hypothesis.timestep as a dictionary.
+
+            ctc_timestamp_type: A str value, which represents the types of timestamps that should be calculated.
+                Can take the following values - "char" for character/subword time stamps, "word" for word level
+                time stamps and "all" (default), for both character level and word level time stamps.
+
+            word_seperator: Str token representing the seperator between words.
+
+            preserve_alignments: Bool flag which preserves the history of logprobs generated during
+                decoding (sample / batched). When set to true, the Hypothesis will contain
+                the non-null value for `logprobs` in it. Here, `logprobs` is a torch.Tensors.
+
+            batch_dim_index: Index of the batch dimension of ``targets`` and ``predictions`` parameters of
+                ``ctc_decoder_predictions_tensor`` methods. Can be either 0 or 1.
+
+            The config may further contain the following sub-dictionaries:
+            "greedy":
+                preserve_alignments: Same as above, overrides above value.
+                compute_timestamps: Same as above, overrides above value.
+
+        tokenizer: NeMo tokenizer object, which inherits from TokenizerSpec.
+    """
+
+    def __init__(self, decoding_cfg, tokenizer: TokenizerSpec):
+        blank_id = tokenizer.tokenizer.vocab_size
+        self.tokenizer = tokenizer
+
+        super().__init__(decoding_cfg=decoding_cfg, blank_id=blank_id)
+
+    def decode_tokens_to_str(self, tokens: List[int]) -> str:
+        """
+        Implemented by subclass in order to decoder a token list into a string.
+
+        Args:
+            tokens: List of int representing the token ids.
+
+        Returns:
+            A decoded string.
+        """
+        hypothesis = self.tokenizer.ids_to_text(tokens)
+        return hypothesis
+
+    def decode_ids_to_tokens(self, tokens: List[int]) -> List[str]:
+        """
+        Implemented by subclass in order to decode a token id list into a token list.
+        A token list is the string representation of each token id.
+
+        Args:
+            tokens: List of int representing the token ids.
+
+        Returns:
+            A list of decoded tokens.
+        """
+        token_list = self.tokenizer.ids_to_tokens(tokens)
+        return token_list
+
+
 class WERBPE(Metric):
     """
     This metric computes numerator and denominator for Overall Word Error Rate for BPE tokens (WER-BPE) between
@@ -48,11 +117,8 @@ class WERBPE(Metric):
             return {'val_loss': val_loss_mean, 'log': tensorboard_logs}
 
     Args:
-        vocabulary: NeMo tokenizer object, which inherits from TokenizerSpec.
-        batch_dim_index: Index of the batch dimension of ``targets`` and ``predictions`` parameters of ``__call__``,
-            ``forward``, ``update``, ``ctc_decoder_predictions_tensor`` methods. Can be either 0 or 1.
+        decoding: An instance of CTCBPEDecoding.
         use_cer: Whether to compute word-error-rate or character-error-rate.
-        ctc_decode: Whether to perform CTC decode.
         log_prediction: Whether to log a single decoded sample per call.
         fold_consecutive: Whether repeated consecutive tokens should be folded into one when decoding.
 
@@ -61,116 +127,27 @@ class WERBPE(Metric):
             distances for all prediction - reference pairs, total number of words in all references.
     """
 
+    full_state_update: bool = True
+
     def __init__(
         self,
-        tokenizer: TokenizerSpec,
-        batch_dim_index=0,
+        decoding: CTCBPEDecoding,
         use_cer=False,
-        ctc_decode=True,
         log_prediction=True,
-        dist_sync_on_step=False,
         fold_consecutive=True,
+        dist_sync_on_step=False,
     ):
         super().__init__(dist_sync_on_step=dist_sync_on_step, compute_on_step=False)
-        self.tokenizer = tokenizer
-        self.batch_dim_index = batch_dim_index
-        self.blank_id = tokenizer.tokenizer.vocab_size
+        self.decoding = decoding
+        self.tokenizer = self.decoding.tokenizer
+        self.blank_id = self.decoding.tokenizer.tokenizer.vocab_size
         self.use_cer = use_cer
-        self.ctc_decode = ctc_decode
         self.log_prediction = log_prediction
         self.fold_consecutive = fold_consecutive
 
         self.add_state("scores", default=torch.tensor(0), dist_reduce_fx='sum', persistent=False)
         self.add_state("words", default=torch.tensor(0), dist_reduce_fx='sum', persistent=False)
 
-    def ctc_decoder_predictions_tensor(
-        self, predictions: torch.Tensor, predictions_len: torch.Tensor = None, return_hypotheses: bool = False
-    ) -> List[str]:
-        """
-        Decodes a sequence of labels to words
-
-        Args:
-            predictions: An integer torch.Tensor of shape [Batch, Time] (if ``batch_index_dim == 0``) or [Time, Batch]
-                (if ``batch_index_dim == 1``) of integer indices that correspond to the index of some character in the
-                label set.
-            predictions_len: Optional tensor of length `Batch` which contains the integer lengths
-                of the sequence in the padded `predictions` tensor.
-            return_hypotheses: Bool flag whether to return just the decoding predictions of the model
-                or a Hypothesis object that holds information such as the decoded `text`,
-                the `alignment` of emited by the CTC Model, and the `length` of the sequence (if available).
-                May also contain the log-probabilities of the decoder (if this method is called via
-                transcribe()) inside `y_sequence`, otherwise it is set None as it is a duplicate of
-                `alignments`.
-
-        Returns:
-            Either a list of str which represent the CTC decoded strings per sample,
-            or a list of Hypothesis objects containing additional information.
-        """
-        hypotheses = []
-        # Drop predictions to CPU
-        predictions = move_dimension_to_the_front(predictions, self.batch_dim_index)
-        prediction_cpu_tensor = predictions.long().cpu()
-        # iterate over batch
-        for ind in range(prediction_cpu_tensor.shape[0]):
-            if self.fold_consecutive:
-                prediction = prediction_cpu_tensor[ind].detach().numpy().tolist()
-                if predictions_len is not None:
-                    prediction = prediction[: predictions_len[ind]]
-                # CTC decoding procedure
-                decoded_prediction = []
-                previous = self.blank_id
-                for p in prediction:
-                    if (p != previous or previous == self.blank_id) and p != self.blank_id:
-                        decoded_prediction.append(p)
-                    previous = p
-            else:
-                prediction = prediction_cpu_tensor[ind].detach()
-                if predictions_len is not None:
-                    prediction = prediction[: predictions_len[ind]]
-                decoded_prediction = prediction[prediction != self.blank_id].tolist()
-
-            text = self.decode_tokens_to_str(decoded_prediction)
-
-            if not return_hypotheses:
-                hypothesis = text
-            else:
-                hypothesis = Hypothesis(
-                    y_sequence=None,  # logprob info added by transcribe method
-                    score=-1.0,
-                    text=text,
-                    alignments=prediction,
-                    length=predictions_len[ind] if predictions_len is not None else 0,
-                )
-            hypotheses.append(hypothesis)
-        return hypotheses
-
-    def decode_tokens_to_str(self, tokens: List[int]) -> str:
-        """
-        Implemented in order to decoder a token list into a string.
-
-        Args:
-            tokens: List of int representing the token ids.
-
-        Returns:
-            A decoded string.
-        """
-        hypothesis = self.tokenizer.ids_to_text(tokens)
-        return hypothesis
-
-    def decode_ids_to_tokens(self, tokens: List[int]) -> List[str]:
-        """
-        Implemented in order to decode a token id list into a token list.
-        A token list is the string representation of each token id.
-
-        Args:
-            tokens: List of int representing the token ids.
-
-        Returns:
-            A list of decoded tokens.
-        """
-        token_list = self.tokenizer.ids_to_tokens(tokens)
-        return token_list
-
     def update(
         self,
         predictions: torch.Tensor,
@@ -181,7 +158,7 @@ class WERBPE(Metric):
         """
         Updates metric state.
         Args:
-            predictions: an integer torch.Tensor of shape ``[Batch, Time]`` (if ``batch_dim_index == 0``) or
+            predictions: an integer torch.Tensor of shape ``[Batch, Time, {Vocabulary}]`` (if ``batch_dim_index == 0``) or
                 ``[Time, Batch]`` (if ``batch_dim_index == 1``)
             targets: an integer torch.Tensor of shape ``[Batch, Time]`` (if ``batch_dim_index == 0``) or
                 ``[Time, Batch]`` (if ``batch_dim_index == 1``)
@@ -192,21 +169,19 @@ class WERBPE(Metric):
         scores = 0.0
         references = []
         with torch.no_grad():
-            # prediction_cpu_tensor = tensors[0].long().cpu()
             targets_cpu_tensor = targets.long().cpu()
-            targets_cpu_tensor = move_dimension_to_the_front(targets_cpu_tensor, self.batch_dim_index)
             tgt_lenths_cpu_tensor = target_lengths.long().cpu()
 
             # iterate over batch
             for ind in range(targets_cpu_tensor.shape[0]):
                 tgt_len = tgt_lenths_cpu_tensor[ind].item()
                 target = targets_cpu_tensor[ind][:tgt_len].numpy().tolist()
-                reference = self.decode_tokens_to_str(target)
+                reference = self.decoding.decode_tokens_to_str(target)
                 references.append(reference)
-            if self.ctc_decode:
-                hypotheses = self.ctc_decoder_predictions_tensor(predictions, predictions_lengths)
-            else:
-                raise NotImplementedError("Implement me if you need non-CTC decode on predictions")
+
+            hypotheses, _ = self.decoding.ctc_decoder_predictions_tensor(
+                predictions, predictions_lengths, fold_consecutive=self.fold_consecutive
+            )
 
         if self.log_prediction:
             logging.info(f"\n")
@@ -232,3 +207,8 @@ class WERBPE(Metric):
         scores = self.scores.detach().float()
         words = self.words.detach().float()
         return scores / words, scores, words
+
+
+@dataclass
+class CTCBPEDecodingConfig(CTCDecodingConfig):
+    pass
