@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import copy
 import itertools
 from math import ceil
 from typing import Dict, List, Optional, Union
@@ -21,7 +20,6 @@ import librosa
 import numpy as np
 import torch
 from omegaconf import DictConfig
-from omegaconf.omegaconf import open_dict
 from pytorch_lightning import Trainer
 from tqdm import tqdm
 
@@ -277,6 +275,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
 
         self.log('loss', loss)
         self.log('learning_rate', self._optimizer.param_groups[0]['lr'])
+        self.log('global_step', self.trainer.global_step)
 
         self._accuracy(logits=logits, labels=labels)
         top_k = self._accuracy.compute()
@@ -354,61 +353,6 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
             'test_acc_top_k': topk_scores,
         }
 
-    def setup_finetune_model(self, model_config: DictConfig):
-        """
-        setup_finetune_model method sets up training data, validation data and test data with new
-        provided config, this checks for the previous labels set up during training from scratch, if None,
-        it sets up labels for provided finetune data from manifest files
-
-        Args:
-            model_config: cfg which has train_ds, optional validation_ds, optional test_ds, 
-            mandatory encoder and decoder model params. Make sure you set num_classes correctly for finetune data.
-
-        Returns: 
-            None
-        """
-        logging.info("Setting up data loaders with manifests provided from model_config")
-
-        if 'train_ds' in model_config and model_config.train_ds is not None:
-            self.setup_training_data(model_config.train_ds)
-        else:
-            raise KeyError("train_ds is not found in model_config but you need it for fine tuning")
-
-        if self.labels is None or len(self.labels) == 0:
-            raise ValueError(f'New labels must be non-empty list of labels. But I got: {self.labels}')
-
-        if 'validation_ds' in model_config and model_config.validation_ds is not None:
-            self.setup_multiple_validation_data(model_config.validation_ds)
-
-        if 'test_ds' in model_config and model_config.test_ds is not None:
-            self.setup_multiple_test_data(model_config.test_ds)
-
-        if self.labels is not None:  # checking for new finetune dataset labels
-            logging.warning(
-                "Trained dataset labels are same as finetune dataset labels -- continuing change of decoder parameters"
-            )
-        else:
-            logging.warning(
-                "Either you provided a dummy manifest file during training from scratch or you restored from a pretrained nemo file"
-            )
-
-        decoder_config = model_config.decoder
-        new_decoder_config = copy.deepcopy(decoder_config)
-        if new_decoder_config['num_classes'] != len(self.labels):
-            raise ValueError(
-                "number of classes provided {} is not same as number of different labels in finetuning data: {}".format(
-                    new_decoder_config['num_classes'], len(self.labels)
-                )
-            )
-
-        del self.decoder
-        self.decoder = EncDecSpeakerLabelModel.from_config_dict(new_decoder_config)
-
-        with open_dict(self._cfg.decoder):
-            self._cfg.decoder = new_decoder_config
-
-        logging.info(f"Changed decoder output to # {self.decoder._num_classes} classes.")
-
     @torch.no_grad()
     def get_embedding(self, path2audio_file):
         """
