@@ -19,7 +19,7 @@ import torch
 import torch.distributed
 import torch.nn as nn
 import torch.nn.functional as F
-from omegaconf import MISSING, ListConfig, OmegaConf
+from omegaconf import MISSING, DictConfig, ListConfig, OmegaConf
 
 from nemo.collections.asr.parts.submodules.jasper import (
     JasperBlock,
@@ -35,6 +35,7 @@ from nemo.collections.asr.parts.submodules.tdnn_attention import (
     TDNNModule,
     TDNNSEModule,
 )
+from nemo.collections.asr.parts.utils import adapter_utils
 from nemo.core.classes.common import typecheck
 from nemo.core.classes.exportable import Exportable
 from nemo.core.classes.mixins import adapter_mixins
@@ -396,7 +397,7 @@ class ParallelConvASREncoder(NeuralModule, Exportable):
         return s_input[-1], length
 
 
-class ConvASRDecoder(NeuralModule, Exportable):
+class ConvASRDecoder(NeuralModule, Exportable, adapter_mixins.AdapterModuleMixin):
     """Simple ASR Decoder for use with CTC-based models such as JasperNet and QuartzNet
 
      Based on these papers:
@@ -442,6 +443,12 @@ class ConvASRDecoder(NeuralModule, Exportable):
 
     @typecheck()
     def forward(self, encoder_output):
+        # Adapter module forward step
+        if self.is_adapter_available():
+            encoder_output = encoder_output.transpose(1, 2)  # [B, T, C]
+            encoder_output = self.forward_enabled_adapters(encoder_output)
+            encoder_output = encoder_output.transpose(1, 2)  # [B, C, T]
+
         return torch.nn.functional.log_softmax(self.decoder_layers(encoder_output).transpose(1, 2), dim=-1)
 
     def input_example(self, max_batch=1, max_dim=256):
@@ -463,6 +470,17 @@ class ConvASRDecoder(NeuralModule, Exportable):
             logging.warning(f"Turned off {m_count} masked convolutions")
         Exportable._prepare_for_export(self, **kwargs)
 
+    # Adapter method overrides
+    def add_adapter(self, name: str, cfg: DictConfig):
+        # Update the config with correct input dim
+        cfg = self._update_adapter_cfg_input_dim(cfg)
+        # Add the adapter
+        super().add_adapter(name=name, cfg=cfg)
+
+    def _update_adapter_cfg_input_dim(self, cfg: DictConfig):
+        cfg = adapter_utils.update_adapter_cfg_input_dim(self, cfg, module_dim=self._feat_in)
+        return cfg
+
     @property
     def vocabulary(self):
         return self.__vocabulary
@@ -482,19 +500,23 @@ class ConvASRDecoderReconstruction(NeuralModule, Exportable):
 
     @property
     def output_types(self):
-        return OrderedDict({"spec_recon": NeuralType(('B', 'T', 'D'), SpectrogramType())})
+        if self.apply_softmax:
+            return OrderedDict({"out": NeuralType(('B', 'T', 'D'), LogprobsType())})
+        else:
+            return OrderedDict({"out": NeuralType(('B', 'T', 'D'), AcousticEncodedRepresentation())})
 
     def __init__(
         self,
         feat_in,
         feat_out,
         feat_hidden,
-        stride_layers,
+        stride_layers=0,
         non_stride_layers=0,
         kernel_size=11,
         init_mode="xavier_uniform",
         activation="relu",
         stride_transpose=True,
+        apply_softmax=False,
     ):
         super().__init__()
 
@@ -556,12 +578,16 @@ class ConvASRDecoderReconstruction(NeuralModule, Exportable):
         self.decoder_layers.append(nn.Conv1d(self.feat_hidden, self.feat_out, kernel_size=1, bias=True))
 
         self.decoder_layers = nn.Sequential(*self.decoder_layers)
+        self.apply_softmax = apply_softmax
 
         self.apply(lambda x: init_weights(x, mode=init_mode))
 
     @typecheck()
     def forward(self, encoder_output):
-        return self.decoder_layers(encoder_output).transpose(-2, -1)
+        out = self.decoder_layers(encoder_output).transpose(-2, -1)
+        if self.apply_softmax:
+            out = torch.nn.functional.log_softmax(out, dim=-1)
+        return out
 
     def input_example(self, max_batch=1, max_dim=256):
         """
@@ -877,20 +903,8 @@ class ConvASREncoderAdapter(ConvASREncoder, adapter_mixins.AdapterModuleMixin):
         return names
 
     def _update_adapter_cfg_input_dim(self, block: JasperBlock, cfg):
-        if 'in_features' in cfg:
-            in_planes = cfg['in_features']
-
-            if in_planes != block.planes:
-                logging.info(f"Updating Adapter input dim from {in_planes} to {block.planes}")
-                in_planes = block.planes
-
-            cfg['in_features'] = in_planes
-            return cfg
-        else:
-            raise ValueError(
-                f"Failed to infer the input dimension of the Adapter cfg. Provided config : \n"
-                f"{OmegaConf.to_yaml(cfg)}"
-            )
+        cfg = adapter_utils.update_adapter_cfg_input_dim(self, cfg, module_dim=block.planes)
+        return cfg
 
 
 @dataclass
