@@ -12,4 +12,4 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from nemo.collections.common.data.dataset import ConcatDataset
+from nemo.collections.common.data.dataset import ConcatDataset, ConcatMapDataset
