@@ -31,13 +31,14 @@
 # with some modifications.
 
 import collections
-import math
 import os
 import subprocess
 import time
 
 import numpy as np
 import torch
+from omegaconf import OmegaConf
+from omegaconf.dictconfig import DictConfig
 
 from nemo.collections.nlp.data.language_modeling.megatron.base_dataset_utils import (
     get_datasets_weights_and_num_samples,
@@ -45,6 +46,7 @@ from nemo.collections.nlp.data.language_modeling.megatron.base_dataset_utils imp
 )
 from nemo.collections.nlp.data.language_modeling.megatron.blendable_dataset import BlendableDataset
 from nemo.collections.nlp.data.language_modeling.megatron.indexed_dataset import make_dataset as make_indexed_dataset
+from nemo.collections.nlp.data.language_modeling.megatron.length_distribution_type import LengthDistribution
 from nemo.collections.nlp.data.language_modeling.megatron.lm_adapted_t5_dataset import T5LMAdaptedDataset
 from nemo.utils import logging
 from nemo.utils.get_rank import is_global_rank_zero
@@ -64,8 +66,9 @@ DSET_TYPE_ICT = 'ict'
 DSET_TYPE_T5 = 't5'
 DSET_TYPE_T5_LM = 't5_prefix_lm'
 DSET_TYPE_BART = 'bart'
+DSET_TYPE_UL2 = 'ul2'
 
-DSET_TYPES = [DSET_TYPE_BERT, DSET_TYPE_ICT, DSET_TYPE_T5, DSET_TYPE_T5_LM, DSET_TYPE_BART]
+DSET_TYPES = [DSET_TYPE_BERT, DSET_TYPE_ICT, DSET_TYPE_T5, DSET_TYPE_T5_LM, DSET_TYPE_BART, DSET_TYPE_UL2]
 
 
 def compile_helper():
@@ -386,6 +389,117 @@ def create_masked_lm_predictions(
     return (output_tokens, masked_lm_positions, masked_lm_labels, token_boundary, masked_spans)
 
 
+def create_extreme_masked_lm_predictions(
+    tokens,
+    masked_lm_prob,
+    mask_id,
+    max_predictions_per_seq,
+    np_rng,
+    max_ngram_size=10,
+    min_ngram_size=2,
+    mean_ngram_size=5,
+    span_length_distribution=LengthDistribution.uniform,
+):
+    """Creates the predictions for the extreme span-masking UL2 objective.
+    Note: Tokens here are vocab ids and not text tokens."""
+    output_tokens = list(tokens)
+
+    masked_lm_positions = []
+    masked_lm_labels = []
+
+    num_to_predict = min(max_predictions_per_seq, max(1, int(round(len(tokens) * masked_lm_prob))))
+    # If the number of tokens to predict is less than the min ngram size, clam it to max predictions.
+    min_ngram_size = int(min(num_to_predict, min_ngram_size))
+
+    ngrams = np.arange(min_ngram_size, max_ngram_size + 1, dtype=np.int64)
+    if span_length_distribution == "uniform":
+        pvals = np.array([1.0 / (max_ngram_size - min_ngram_size + 1)] * (max_ngram_size - min_ngram_size + 1))
+
+    ngram_indexes = []
+    cand_indexes = [[i] for i in range(len(tokens))]
+    for idx in range(len(cand_indexes)):
+        ngram_index = []
+        for n in ngrams:
+            ngram_index.append(cand_indexes[idx : idx + n])
+        ngram_indexes.append(ngram_index)
+
+    np_rng.shuffle(ngram_indexes)
+
+    (masked_lms, masked_spans) = ([], [])
+    covered_indexes = set()
+    for cand_index_set in ngram_indexes:
+        if len(masked_lms) >= num_to_predict:
+            break
+        if not cand_index_set:
+            continue
+        # Note(mingdachen):
+        # Skip current piece if they are covered in lm masking or previous ngrams.
+        for index_set in cand_index_set[0]:
+            for index in index_set:
+                if index in covered_indexes:
+                    continue
+
+        if span_length_distribution == LengthDistribution.uniform:
+            n = np_rng.choice(
+                ngrams[: len(cand_index_set)],
+                p=pvals[: len(cand_index_set)] / pvals[: len(cand_index_set)].sum(keepdims=True),
+            )
+        elif span_length_distribution == LengthDistribution.geometric:
+            # Sampling "n" from the geometric distribution and clipping it to
+            # the max_ngrams. Using p=0.2 default from the SpanBERT paper
+            # https://arxiv.org/pdf/1907.10529.pdf (Sec 3.1)
+
+            # The expectation of a geometric distribution is E[X] = 1 / p
+            p = 1 / mean_ngram_size if mean_ngram_size is not None else 0.2
+            n = np_rng.geometric(p)
+            n = int(np.clip(n, min_ngram_size, max_ngram_size))
+        elif span_length_distribution == LengthDistribution.truncated_normal:
+            # Sampling "n" from a truncated normal distribution.
+            mu = mean_ngram_size if mean_ngram_size is not None else (max_ngram_size - min_ngram_size) // 2
+            n = int(np.clip(np_rng.normal(loc=mu, scale=np.sqrt(mu)), min_ngram_size, max_ngram_size))
+
+        index_set = sum(cand_index_set[n - min_ngram_size], [])
+        n -= 1
+        # Note(mingdachen):
+        # Repeatedly looking for a candidate that does not exceed the
+        # maximum number of predictions by trying shorter ngrams.
+        while len(masked_lms) + len(index_set) > num_to_predict:
+            if n < min_ngram_size:
+                break
+            index_set = sum(cand_index_set[n - min_ngram_size], [])
+            n -= 1
+
+        # If adding a whole-word mask would exceed the maximum number of
+        # predictions, then just skip this candidate.
+        if len(masked_lms) + len(index_set) > num_to_predict:
+            continue
+        is_any_index_covered = False
+        for index in index_set:
+            if index in covered_indexes:
+                is_any_index_covered = True
+                break
+        if is_any_index_covered:
+            continue
+        for index in index_set:
+            covered_indexes.add(index)
+            output_tokens[index] = mask_id
+            masked_lms.append(MaskedLmInstance(index=index, label=tokens[index]))
+
+        masked_spans.append(MaskedLmInstance(index=index_set, label=[tokens[index] for index in index_set]))
+
+    assert len(masked_lms) <= num_to_predict
+    np_rng.shuffle(ngram_indexes)
+
+    masked_lms = sorted(masked_lms, key=lambda x: x.index)
+    # Sort the spans by the index of the first span
+    masked_spans = sorted(masked_spans, key=lambda x: x.index[0])
+
+    for p in masked_lms:
+        masked_lm_positions.append(p.index)
+        masked_lm_labels.append(p.label)
+    return (output_tokens, masked_lm_positions, masked_lm_labels, masked_spans)
+
+
 def pad_and_convert_to_numpy(tokens, tokentypes, masked_positions, masked_labels, pad_id, max_seq_length):
     """Pad sequences and convert them to numpy."""
 
@@ -417,6 +531,14 @@ def pad_and_convert_to_numpy(tokens, tokentypes, masked_positions, masked_labels
     return tokens_np, tokentypes_np, labels_np, padding_mask_np, loss_mask_np
 
 
+def make_text_memmap_bin_compatibility(text_memmap_ds):
+    """Make a TextMemMapDataset compatible with binary memmap."""
+    text_memmap_ds.doc_idx = np.arange(len(text_memmap_ds) + 1, dtype=np.int64)
+    text_memmap_ds.sizes = np.ones(len(text_memmap_ds), dtype=np.int32)
+
+    return text_memmap_ds
+
+
 def build_train_valid_test_datasets(
     cfg,
     trainer,
@@ -440,7 +562,24 @@ def build_train_valid_test_datasets(
     whole_word_masking=True,
     favor_long_ngrams=False,
     delete_mask_prob=0,
+    respect_document_boundaries=True,
+    data_impl_kwargs={},
 ):
+    # for VSC and text memmap we need to provide a tokenizer, if not given
+    if data_impl in ["text_mmap", "csv_mmap"]:
+        if "tokenizer" not in data_impl_kwargs:
+            if isinstance(data_impl_kwargs, DictConfig):
+                data_impl_kwargs = OmegaConf.to_object(data_impl_kwargs)
+            else:
+                # prevent updating the default
+                data_impl_kwargs = data_impl_kwargs.copy()
+
+            data_impl_kwargs["tokenizer"] = tokenizer
+
+    if not respect_document_boundaries and data_impl_kwargs != {}:
+        raise ValueError(
+            "respect_document_boundaries=False is not compatible with text_memmap and csv_memmap (data_impl_kwargs != {})"
+        )
 
     if len(data_prefix) == 1:
         return _build_train_valid_test_datasets(
@@ -466,11 +605,14 @@ def build_train_valid_test_datasets(
             whole_word_masking=whole_word_masking,
             favor_long_ngrams=favor_long_ngrams,
             delete_mask_prob=delete_mask_prob,
+            respect_document_boundaries=respect_document_boundaries,
+            data_impl_kwargs=data_impl_kwargs,
         )
     # Blending dataset.
     # Parse the values.
     output = get_datasets_weights_and_num_samples(data_prefix, train_valid_test_num_samples)
     prefixes, weights, datasets_train_valid_test_num_samples = output
+    train_n, valid_n, test_n = map(sum, zip(*datasets_train_valid_test_num_samples))
 
     # Build individual datasets.
     train_datasets = []
@@ -500,6 +642,8 @@ def build_train_valid_test_datasets(
             whole_word_masking=whole_word_masking,
             favor_long_ngrams=favor_long_ngrams,
             delete_mask_prob=delete_mask_prob,
+            respect_document_boundaries=respect_document_boundaries,
+            data_impl_kwargs=data_impl_kwargs,
         )
         if train_ds:
             train_datasets.append(train_ds)
@@ -511,13 +655,13 @@ def build_train_valid_test_datasets(
         # Blend.
     blending_train_dataset = None
     if train_datasets:
-        blending_train_dataset = BlendableDataset(train_datasets, weights)
+        blending_train_dataset = BlendableDataset(train_datasets, weights, train_n)
     blending_valid_dataset = None
     if valid_datasets:
-        blending_valid_dataset = BlendableDataset(valid_datasets, weights)
+        blending_valid_dataset = BlendableDataset(valid_datasets, weights, valid_n)
     blending_test_dataset = None
     if test_datasets:
-        blending_test_dataset = BlendableDataset(test_datasets, weights)
+        blending_test_dataset = BlendableDataset(test_datasets, weights, test_n)
 
     return (blending_train_dataset, blending_valid_dataset, blending_test_dataset)
 
@@ -545,16 +689,18 @@ def _build_train_valid_test_datasets(
     whole_word_masking=True,
     favor_long_ngrams=False,
     delete_mask_prob=0,  # This flag is used in BART only, and will not have effect on T5/BERT
+    respect_document_boundaries=True,
+    data_impl_kwargs={},
 ):
 
     if dataset_type not in DSET_TYPES:
         raise ValueError("Invalid dataset_type: ", dataset_type)
 
     # Indexed dataset.
-    indexed_dataset = get_indexed_dataset_(data_prefix, data_impl, skip_warmup)
+    indexed_dataset = get_indexed_dataset_(data_prefix, data_impl, skip_warmup, data_impl_kwargs=data_impl_kwargs)
 
-    if dataset_type == DSET_TYPE_ICT:
-        title_dataset = get_indexed_dataset_(args.titles_data_path, data_impl, skip_warmup)
+    # if dataset_type == DSET_TYPE_ICT:
+    #     title_dataset = get_indexed_dataset_(args.titles_data_path, data_impl, skip_warmup)
 
     # Get start and end indices of train/valid/train into doc-idx
     # Note that doc-idx is desinged to be num-docs + 1 so we can
@@ -586,18 +732,22 @@ def _build_train_valid_test_datasets(
         # from nemo.collections.nlp.data.language_modeling.megatron.ict_dataset import ICTDataset
         from nemo.collections.nlp.data.language_modeling.megatron.bert_dataset import BertDataset
         from nemo.collections.nlp.data.language_modeling.megatron.t5_dataset import T5Dataset
+        from nemo.collections.nlp.data.language_modeling.megatron.ul2_dataset import UL2Dataset
         from nemo.collections.nlp.data.language_modeling.megatron.bart_dataset import BARTDataset
+        from nemo.collections.nlp.data.language_modeling.megatron.length_distribution_type import LengthDistribution
 
         dataset = None
         if splits[index + 1] > splits[index]:
             # Get the pointer to the original doc-idx so we can set it later.
-            doc_idx_ptr = indexed_dataset.get_doc_idx()
+            if hasattr(indexed_dataset, 'get_doc_idx'):
+                doc_idx_ptr = indexed_dataset.get_doc_idx()
             # Slice the doc-idx
             start_index = splits[index]
             # Add +1 so we can index into the dataset to get the upper bound.
             end_index = splits[index + 1] + 1
             # New doc_idx view.
-            indexed_dataset.set_doc_idx(doc_idx_ptr[start_index:end_index])
+            if hasattr(indexed_dataset, 'set_doc_idx'):
+                indexed_dataset.set_doc_idx(doc_idx_ptr[start_index:end_index])
             # Build the dataset accordingly.
             kwargs = dict(
                 name=name,
@@ -623,6 +773,7 @@ def _build_train_valid_test_datasets(
             elif dataset_type == DSET_TYPE_T5:
                 assert tokenizer is not None, "Tokenizer is required for T5 dataset"
                 logging.info("Instatiating T5 Dataset ...")
+                documents = np.arange(start=splits[index], stop=splits[index + 1], step=1, dtype=np.int32)
                 dataset = T5Dataset(
                     cfg=cfg,
                     trainer=trainer,
@@ -637,6 +788,8 @@ def _build_train_valid_test_datasets(
                     permutation=permutation,
                     whole_word_masking=whole_word_masking,
                     favor_long_ngrams=favor_long_ngrams,
+                    documents=documents,
+                    respect_document_boundaries=respect_document_boundaries,
                     **kwargs,
                 )
             elif dataset_type == DSET_TYPE_BERT:
@@ -660,10 +813,13 @@ def _build_train_valid_test_datasets(
                     documents=documents,
                     indexed_dataset=indexed_dataset,
                     num_samples=int(train_valid_test_num_samples[index]),
+                    max_seq_length_encoder=max_seq_length,
+                    max_seq_length_decoder=max_seq_length_dec,
                     **kwargs,
                 )
             elif dataset_type == DSET_TYPE_BART:
                 assert tokenizer is not None, "Tokenizer is required for BART dataset"
+                documents = np.arange(start=splits[index], stop=splits[index + 1], step=1, dtype=np.int32)
                 logging.info("Instatiating BART Dataset ...")
                 dataset = BARTDataset(
                     cfg=cfg,
@@ -679,13 +835,62 @@ def _build_train_valid_test_datasets(
                     whole_word_masking=whole_word_masking,
                     favor_long_ngrams=favor_long_ngrams,
                     delete_mask_prob=delete_mask_prob,
+                    documents=documents,
+                    respect_document_boundaries=respect_document_boundaries,
+                    **kwargs,
+                )
+            elif dataset_type == DSET_TYPE_UL2:
+                assert tokenizer is not None, "Tokenizer is required for UL2 dataset"
+                documents = np.arange(start=splits[index], stop=splits[index + 1], step=1, dtype=np.int32)
+                logging.info("Instatiating UL2 Dataset ...")
+                extreme_ngram_span_length_distribution = cfg.data.get(
+                    "extreme_ngram_span_length_distribution", "truncated_normal"
+                )
+                ngram_span_length_distribution = cfg.data.get("ngram_span_length_distribution", "geometric")
+                if extreme_ngram_span_length_distribution == "truncated_normal":
+                    extreme_ngram_span_length_distribution = LengthDistribution.truncated_normal
+                elif extreme_ngram_span_length_distribution == "uniform":
+                    extreme_ngram_span_length_distribution = LengthDistribution.uniform
+                elif extreme_ngram_span_length_distribution == "geometric":
+                    extreme_ngram_span_length_distribution = LengthDistribution.geometric
+
+                if ngram_span_length_distribution == "truncated_normal":
+                    ngram_span_length_distribution = LengthDistribution.truncated_normal
+                elif ngram_span_length_distribution == "uniform":
+                    ngram_span_length_distribution = LengthDistribution.uniform
+                elif ngram_span_length_distribution == "geometric":
+                    ngram_span_length_distribution = LengthDistribution.geometric
+
+                dataset = UL2Dataset(
+                    cfg=cfg,
+                    trainer=trainer,
+                    tokenizer=tokenizer,
+                    indexed_dataset=indexed_dataset,
+                    masked_lm_prob=masked_lm_prob,
+                    max_seq_length_dec=max_seq_length_dec,
+                    short_seq_prob=short_seq_prob,
+                    max_ngram_size=max_ngram_size,
+                    mean_ngram_size=mean_ngram_size,
+                    ngram_span_length_distribution=ngram_span_length_distribution,
+                    extreme_ngram_span_length_distribution=extreme_ngram_span_length_distribution,
+                    permutation=permutation,
+                    whole_word_masking=whole_word_masking,
+                    favor_long_ngrams=favor_long_ngrams,
+                    extreme_masked_lm_prob=cfg.data.get("extreme_masked_lm_prob", 0.5),
+                    extreme_max_ngram_size=cfg.data.get("extreme_max_ngram_size", 128),
+                    extreme_mean_ngram_size=cfg.data.get("extreme_mean_ngram_size", 64),
+                    extreme_min_ngram_size=cfg.data.get("extreme_min_ngram_size", 32),
+                    prefix_lm_pivot_mean=cfg.data.get("prefix_lm_pivot_mean", 0.25),
+                    respect_document_boundaries=respect_document_boundaries,
+                    documents=documents,
                     **kwargs,
                 )
             else:
                 raise NotImplementedError("Dataset type not fully implemented.")
 
             # Set the original pointer so dataset remains the main dataset.
-            indexed_dataset.set_doc_idx(doc_idx_ptr)
+            if hasattr(indexed_dataset, 'set_doc_idx'):
+                indexed_dataset.set_doc_idx(doc_idx_ptr)
             # Checks.
             assert indexed_dataset.doc_idx[0] == 0
             assert indexed_dataset.doc_idx.shape[0] == (total_num_of_documents + 1)
@@ -698,12 +903,16 @@ def _build_train_valid_test_datasets(
     return (train_dataset, valid_dataset, test_dataset)
 
 
-def get_indexed_dataset_(data_prefix, data_impl, skip_warmup):
+def get_indexed_dataset_(data_prefix, data_impl, skip_warmup, data_impl_kwargs={}):
 
     logging.info(' > building dataset index ...')
 
     start_time = time.time()
-    indexed_dataset = make_indexed_dataset(data_prefix, data_impl, skip_warmup)
+    indexed_dataset = make_indexed_dataset(data_prefix, data_impl, skip_warmup, impl_kwargs=data_impl_kwargs)
+    if data_impl in ['text_mmap', 'csv_mmap']:
+        # make csv/text memmap compatible with Megatron sampling
+        make_text_memmap_bin_compatibility(indexed_dataset)
+
     assert indexed_dataset.sizes.shape[0] == indexed_dataset.doc_idx[-1]
     logging.info(' > finished creating indexed dataset in {:4f} ' 'seconds'.format(time.time() - start_time))
 
