@@ -14,13 +14,11 @@
 import string
 import sys
 from copy import deepcopy
-from unicodedata import category
 
 import regex as re
 from tqdm import tqdm
 
 from nemo.collections.nlp.data.text_normalization import constants
-from nemo.utils import logging
 
 __all__ = [
     'read_data_file',
@@ -194,67 +192,3 @@ def normalize_str(input_str):
 def remove_puncts(input_str):
     """ Remove punctuations from an input string """
     return input_str.translate(str.maketrans('', '', string.punctuation))
-
-
-def post_process_punct(input: str, normalized_text: str):
-    """
-    Post-processing of the normalized output to match input in terms of spaces around punctuation marks.
-    After NN normalization, Moses detokenization puts a space after
-    punctuation marks, and attaches an opening quote "'" to the word to the right.
-    E.g., input to the TN NN model is "12 test' example",
-    after normalization and detokenization -> "twelve test 'example" (the quote is considered to be an opening quote,
-    but it doesn't match the input and can cause issues during TTS voice generation.)
-    The current function will match the punctuation and spaces of the normalized text with the input sequence.
-    "12 test' example" -> "twelve test 'example" -> "twelve test' example" (the quote was shifted to match the input).
-
-    Args:
-        input: input text (original input to the NN, before normalization or tokenization)
-        normalized_text: output text (output of the TN NN model)
-    """
-    input = [x for x in input]
-    normalized_text = [x for x in normalized_text]
-    punct_default = [x for x in string.punctuation]
-    punct_unicode = [chr(i) for i in range(sys.maxunicode) if category(chr(i)).startswith("P")]
-    punct_marks = set(punct_default + punct_unicode)
-    for punct in punct_marks:
-        try:
-            equal = True
-            if input.count(punct) != normalized_text.count(punct):
-                equal = False
-            idx_in, idx_out = 0, 0
-            while punct in input[idx_in:]:
-                idx_out = normalized_text.index(punct, idx_out)
-                idx_in = input.index(punct, idx_in)
-
-                def _is_valid(idx_out, idx_in, normalized_text, input):
-                    """Check if previous or next word match (for cases when punctuation marks are part of
-                    semiotic token, i.e. some punctuation can be missing in the normalized text)"""
-                    return (idx_out > 0 and idx_in > 0 and normalized_text[idx_out - 1] == input[idx_in - 1]) or (
-                        idx_out < len(normalized_text) - 1
-                        and idx_in < len(input) - 1
-                        and normalized_text[idx_out + 1] == input[idx_in + 1]
-                    )
-
-                if not equal and not _is_valid(idx_out, idx_in, normalized_text, input):
-                    idx_in += 1
-                    continue
-
-                if idx_in > 0 and idx_out > 0:
-                    if normalized_text[idx_out - 1] == " " and input[idx_in - 1] != " ":
-                        normalized_text[idx_out - 1] = ""
-
-                    elif normalized_text[idx_out - 1] != " " and input[idx_in - 1] == " ":
-                        normalized_text[idx_out - 1] += " "
-
-                if idx_in < len(input) - 1 and idx_out < len(normalized_text) - 1:
-                    if normalized_text[idx_out + 1] == " " and input[idx_in + 1] != " ":
-                        normalized_text[idx_out + 1] = ""
-                    elif normalized_text[idx_out + 1] != " " and input[idx_in + 1] == " ":
-                        normalized_text[idx_out] = normalized_text[idx_out] + " "
-                idx_out += 1
-                idx_in += 1
-        except:
-            logging.debug(f"Skipping post-processing of {''.join(normalized_text)} for '{punct}'")
-
-    normalized_text = "".join(normalized_text)
-    return re.sub(r' +', ' ', normalized_text)
