@@ -123,7 +123,10 @@ class GreedySequenceGenerator:
         # is min(max_sequence_length, src_len + max_delta_length)
         if encoder_hidden_states is not None:
             batch_size, src_len, _ = encoder_hidden_states.size()
-            max_seq_length = min(self.max_seq_length, src_len + self.max_delta_len)
+            if self.max_delta_len >= 0:
+                max_seq_length = min(self.max_seq_length, src_len + self.max_delta_len)
+            else:
+                max_seq_length = self.max_seq_length
         else:
             max_seq_length = self.max_seq_length
 
@@ -511,7 +514,10 @@ class EnsembleBeamSearchSequenceGenerator:
         # is min(max_sequence_length, src_len + max_delta_length)
         if encoder_hidden_states is not None:
             batch_size, src_len, _ = encoder_hidden_states.size()
-            max_seq_length = min(self.max_seq_length, src_len + self.max_delta_len)
+            if self.max_delta_len >= 0:
+                max_seq_length = min(self.max_seq_length, src_len + self.max_delta_len)
+            else:
+                max_seq_length = self.max_seq_length
         else:
             max_seq_length = self.max_seq_length
 
