@@ -16,6 +16,7 @@ from nemo.collections.tts.models.aligner import AlignerModel
 from nemo.collections.tts.models.fastpitch import FastPitchModel
 from nemo.collections.tts.models.hifigan import HifiGanModel
 from nemo.collections.tts.models.mixer_tts import MixerTTSModel
+from nemo.collections.tts.models.radtts import RadTTSModel
 from nemo.collections.tts.models.tacotron2 import Tacotron2Model
 from nemo.collections.tts.models.two_stages import GriffinLimModel, MelPsuedoInverseModel, TwoStagesModel
 from nemo.collections.tts.models.univnet import UnivNetModel
@@ -32,4 +33,5 @@ __all__ = [
     "AlignerModel",
     "MixerTTSModel",
     "UnivNetModel",
+    "RadTTSModel",
 ]
