@@ -47,8 +47,8 @@ from nemo.utils import logging, model_utils
 @dataclass
 class G2PConfig:
     _target_: str = "nemo.collections.tts.torch.g2ps.EnglishG2p"
-    phoneme_dict: str = "scripts/tts_dataset_files/cmudict-0.7b_nv22.01"
-    heteronyms: str = "scripts/tts_dataset_files/heteronyms-030921"
+    phoneme_dict: str = "scripts/tts_dataset_files/cmudict-0.7b_nv22.08"
+    heteronyms: str = "scripts/tts_dataset_files/heteronyms-052722"
     phoneme_probability: float = 0.5
 
 
@@ -152,6 +152,7 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             cfg.n_mel_channels,
         )
         self._input_types = self._output_types = None
+        self.export_config = {"enable_volume": False, "enable_ragged_batches": False}
 
     def _get_default_text_tokenizer_conf(self):
         text_tokenizer: TextTokenizerConfig = TextTokenizerConfig()
@@ -166,7 +167,14 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
                     'text_normalizer.whitelist', cfg.text_normalizer.whitelist
                 )
 
-            self.normalizer = instantiate(cfg.text_normalizer, **normalizer_kwargs)
+            try:
+                self.normalizer = instantiate(cfg.text_normalizer, **normalizer_kwargs)
+            except Exception as e:
+                logging.error(e)
+                raise ImportError(
+                    "`pynini` not installed, please install via NeMo/nemo_text_processing/pynini_install.sh"
+                )
+
             self.text_normalizer_call = self.normalizer.normalize
             if "text_normalizer_call_kwargs" in cfg:
                 self.text_normalizer_call_kwargs = cfg.text_normalizer_call_kwargs
@@ -463,7 +471,7 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
                     cfg.dataloader_params.shuffle = True
             elif not cfg.dataloader_params.shuffle:
                 logging.error(f"The {name} dataloader for {self} has shuffle set to False!!!")
-        elif not shuffle_should_be and cfg.dataloader_params.shuffle:
+        elif cfg.dataloader_params.shuffle:
             logging.error(f"The {name} dataloader for {self} has shuffle set to True!!!")
 
         if cfg.dataset._target_ == "nemo.collections.tts.torch.data.TTSDataset":
@@ -501,6 +509,8 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
             List of available pre-trained models.
         """
         list_of_models = []
+
+        # en-US, single speaker, 22050Hz, LJSpeech.
         model = PretrainedModelInfo(
             pretrained_model_name="tts_en_fastpitch",
             location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_en_fastpitch/versions/1.8.1/files/tts_en_fastpitch_align.nemo",
@@ -509,26 +519,59 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         )
         list_of_models.append(model)
 
+        # en-US, multi-speaker, 44100Hz, HiFiTTS.
+        model = PretrainedModelInfo(
+            pretrained_model_name="tts_en_fastpitch_multispeaker",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_en_multispeaker_fastpitchhifigan/versions/1.10.0/files/tts_en_fastpitch_multispeaker.nemo",
+            description="This model is trained on HiFITTS sampled at 44100Hz with and can be used to generate male and female English voices with an American accent.",
+            class_=cls,
+        )
+        list_of_models.append(model)
+
+        # de-DE, single speaker, 22050 Hz, OpenSLR Neutral German Dataset.
+        model = PretrainedModelInfo(
+            pretrained_model_name="tts_de_fastpitch_singlespeaker",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_de_fastpitchhifigan/versions/1.10.0/files/tts_de_fastpitch_align.nemo",
+            description="This model is trained on a single male speaker data in OpenSLR Neutral German Dataset sampled at 22050Hz and can be used to generate male German voices.",
+            class_=cls,
+        )
+        list_of_models.append(model)
+
+        # de-DE, multi-speaker, 5 speakers, 44100 Hz, HUI-Audio-Corpus-German Clean.
+        model = PretrainedModelInfo(
+            pretrained_model_name="tts_de_fastpitch_multispeaker_5",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_de_fastpitch_multispeaker_5/versions/1.11.0/files/tts_de_fastpitch_multispeaker_5.nemo",
+            description="This model is trained on 5 speakers in HUI-Audio-Corpus-German clean subset sampled at 44100Hz with and can be used to generate male and female German voices.",
+            class_=cls,
+        )
+        list_of_models.append(model)
+
         return list_of_models
 
     # Methods for model exportability
     def _prepare_for_export(self, **kwargs):
         super()._prepare_for_export(**kwargs)
 
+        tensor_shape = ('T') if self.export_config["enable_ragged_batches"] else ('B', 'T')
+
         # Define input_types and output_types as required by export()
         self._input_types = {
-            "text": NeuralType(('B', 'T_text'), TokenIndex()),
-            "pitch": NeuralType(('B', 'T_text'), RegressionValuesType()),
-            "pace": NeuralType(('B', 'T_text'), optional=True),
-            "speaker": NeuralType(('B'), Index()),
+            "text": NeuralType(tensor_shape, TokenIndex()),
+            "pitch": NeuralType(tensor_shape, RegressionValuesType()),
+            "pace": NeuralType(tensor_shape),
+            "volume": NeuralType(tensor_shape, optional=True),
+            "batch_lengths": NeuralType(('B'), optional=True),
+            "speaker": NeuralType(('B'), Index(), optional=True),
         }
         self._output_types = {
-            "spect": NeuralType(('B', 'D', 'T_spec'), MelSpectrogramType()),
+            "spect": NeuralType(('B', 'D', 'T'), MelSpectrogramType()),
             "num_frames": NeuralType(('B'), TokenDurationType()),
-            "durs_predicted": NeuralType(('B', 'T_text'), TokenDurationType()),
-            "log_durs_predicted": NeuralType(('B', 'T_text'), TokenLogDurationType()),
-            "pitch_predicted": NeuralType(('B', 'T_text'), RegressionValuesType()),
+            "durs_predicted": NeuralType(('B', 'T'), TokenDurationType()),
+            "log_durs_predicted": NeuralType(('B', 'T'), TokenLogDurationType()),
+            "pitch_predicted": NeuralType(('B', 'T'), RegressionValuesType()),
         }
+        if self.export_config["enable_volume"]:
+            self._output_types["volume_aligned"] = NeuralType(('B', 'T'), RegressionValuesType())
 
     def _export_teardown(self):
         self._input_types = self._output_types = None
@@ -539,6 +582,10 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         disabled_inputs = set()
         if self.fastpitch.speaker_emb is None:
             disabled_inputs.add("speaker")
+        if not self.export_config["enable_ragged_batches"]:
+            disabled_inputs.add("batch_lengths")
+        if not self.export_config["enable_volume"]:
+            disabled_inputs.add("volume")
         return disabled_inputs
 
     @property
@@ -549,22 +596,43 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
     def output_types(self):
         return self._output_types
 
-    def input_example(self, max_batch=1, max_dim=256):
+    def input_example(self, max_batch=1, max_dim=44):
         """
         Generates input examples for tracing etc.
         Returns:
             A tuple of input examples.
         """
         par = next(self.fastpitch.parameters())
-        sz = (max_batch, max_dim)
+        sz = (max_batch * max_dim,) if self.export_config["enable_ragged_batches"] else (max_batch, max_dim)
         inp = torch.randint(
             0, self.fastpitch.encoder.word_emb.num_embeddings, sz, device=par.device, dtype=torch.int64
         )
         pitch = torch.randn(sz, device=par.device, dtype=torch.float32) * 0.5
-        pace = torch.clamp((torch.randn(sz, device=par.device, dtype=torch.float32) + 1) * 0.1, min=0.01)
+        pace = torch.clamp(torch.randn(sz, device=par.device, dtype=torch.float32) * 0.1 + 1, min=0.01)
 
         inputs = {'text': inp, 'pitch': pitch, 'pace': pace}
 
+        if self.export_config["enable_volume"]:
+            volume = torch.clamp(torch.randn(sz, device=par.device, dtype=torch.float32) * 0.1 + 1, min=0.01)
+            inputs['volume'] = volume
+        if self.export_config["enable_ragged_batches"]:
+            batch_lengths = torch.zeros((max_batch + 1), device=par.device, dtype=torch.int32)
+            left_over_size = sz[0]
+            batch_lengths[0] = 0
+            for i in range(1, max_batch):
+                length = torch.randint(1, left_over_size - (max_batch - i), (1,), device=par.device)
+                batch_lengths[i] = length + batch_lengths[i - 1]
+                left_over_size -= length.detach().cpu().numpy()[0]
+            batch_lengths[-1] = left_over_size + batch_lengths[-2]
+
+            sum = 0
+            index = 1
+            while index < len(batch_lengths):
+                sum += batch_lengths[index] - batch_lengths[index - 1]
+                index += 1
+            assert sum == sz[0], f"sum: {sum}, sz: {sz[0]}, lengths:{batch_lengths}"
+            inputs['batch_lengths'] = batch_lengths
+
         if self.fastpitch.speaker_emb is not None:
             inputs['speaker'] = torch.randint(
                 0, self.fastpitch.speaker_emb.num_embeddings, (max_batch,), device=par.device, dtype=torch.int64
@@ -572,5 +640,45 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
 
         return (inputs,)
 
-    def forward_for_export(self, text, pitch, pace, speaker=None):
-        return self.fastpitch.infer(text=text, pitch=pitch, pace=pace, speaker=speaker)
+    def forward_for_export(self, text, pitch, pace, volume=None, batch_lengths=None, speaker=None):
+        if self.export_config["enable_ragged_batches"]:
+            text, pitch, pace, volume_tensor = create_batch(
+                text, pitch, pace, batch_lengths, padding_idx=self.fastpitch.encoder.padding_idx, volume=volume
+            )
+            if volume is not None:
+                volume = volume_tensor
+        return self.fastpitch.infer(text=text, pitch=pitch, pace=pace, volume=volume, speaker=speaker)
+
+
+@torch.jit.script
+def create_batch(
+    text: torch.Tensor,
+    pitch: torch.Tensor,
+    pace: torch.Tensor,
+    batch_lengths: torch.Tensor,
+    padding_idx: int = -1,
+    volume: Optional[torch.Tensor] = None,
+):
+    batch_lengths = batch_lengths.to(torch.int64)
+    max_len = torch.max(batch_lengths[1:] - batch_lengths[:-1])
+
+    index = 1
+    texts = torch.zeros(batch_lengths.shape[0] - 1, max_len, dtype=torch.int64, device=text.device) + padding_idx
+    pitches = torch.zeros(batch_lengths.shape[0] - 1, max_len, dtype=torch.float32, device=text.device)
+    paces = torch.zeros(batch_lengths.shape[0] - 1, max_len, dtype=torch.float32, device=text.device) + 1.0
+    volumes = torch.zeros(batch_lengths.shape[0] - 1, max_len, dtype=torch.float32, device=text.device) + 1.0
+
+    while index < batch_lengths.shape[0]:
+        seq_start = batch_lengths[index - 1]
+        seq_end = batch_lengths[index]
+        cur_seq_len = seq_end - seq_start
+
+        texts[index - 1, :cur_seq_len] = text[seq_start:seq_end]
+        pitches[index - 1, :cur_seq_len] = pitch[seq_start:seq_end]
+        paces[index - 1, :cur_seq_len] = pace[seq_start:seq_end]
+        if volume is not None:
+            volumes[index - 1, :cur_seq_len] = volume[seq_start:seq_end]
+
+        index += 1
+
+    return texts, pitches, paces, volumes
