@@ -65,7 +65,7 @@ class GradBucket(object):
     Persistent buffer for main gradients that remains allocated between training iterations
     """
 
-    def __init__(self, numel):
+    def __init__(self, numel, chunk_size_mb):
         if not HAVE_APEX:
             raise ImportError(
                 "Apex was not found. Please see the NeMo README for installation instructions: https://github.com/NVIDIA/NeMo#megatron-gpt."
@@ -74,6 +74,20 @@ class GradBucket(object):
         self.numel = numel
         self.data = torch.zeros(self.numel, dtype=torch.float, device=torch.cuda.current_device(), requires_grad=False)
 
+        self.chunk_size_mb = chunk_size_mb
+        if self.chunk_size_mb > 0:
+            chunk_size_bytes = chunk_size_mb * 1024 * 1024
+            self.chunk_size_numel = chunk_size_bytes // 4
+            self.num_chunks = self.numel // self.chunk_size_numel
+            self.numel_per_chunk = [self.chunk_size_numel] * self.num_chunks
+            if self.numel % self.chunk_size_numel != 0:
+                self.num_chunks += 1
+                self.numel_per_chunk.append(self.numel % self.chunk_size_numel)
+
+            self.start_index_per_chunk = torch.cumsum(torch.tensor([0] + self.numel_per_chunk[:-1]), dim=0)
+            self.current_chunk = 0
+            self.computed_numel_per_chunk = [0] * self.num_chunks
+
     def zero(self):
         """Reset the buffer to zero."""
         self.data.zero_()
@@ -90,7 +104,40 @@ class GradBucket(object):
         assert end_index <= self.numel, 'requested tensor is out of the buffer range.'
         buffer_tensor = self.data[start_index:end_index]
         buffer_tensor = buffer_tensor.view(shape)
-        return buffer_tensor
+
+        grad_chunk_info = None
+        if self.chunk_size_mb > 0:
+            grad_chunk_info = {}
+            chunk = start_index // self.chunk_size_numel
+            chunk_start_index = self.start_index_per_chunk[chunk]
+            chunk_end_index = chunk_start_index + self.numel_per_chunk[chunk]
+            grad_chunk_info[chunk] = min(chunk_end_index, end_index) - start_index
+            while chunk_end_index < end_index:
+                chunk += 1
+                chunk_start_index = self.start_index_per_chunk[chunk]
+                chunk_end_index = chunk_start_index + self.numel_per_chunk[chunk]
+                grad_chunk_info[chunk] = min(chunk_end_index, end_index) - chunk_start_index
+
+        return buffer_tensor, grad_chunk_info
+
+    def update_chunk_info(self, grad_chunk_info):
+        for chunk in grad_chunk_info.keys():
+            self.computed_numel_per_chunk[chunk] += grad_chunk_info[chunk]
+
+    def get_allreduce_tensor(self):
+        if self.computed_numel_per_chunk[self.current_chunk] == self.numel_per_chunk[self.current_chunk]:
+            chunk_start_index = self.start_index_per_chunk[self.current_chunk]
+            chunk_end_index = chunk_start_index + self.numel_per_chunk[self.current_chunk]
+            allreduce_tensor = self.data[chunk_start_index:chunk_end_index]
+
+            self.computed_numel_per_chunk[self.current_chunk] = 0
+            self.current_chunk += 1
+            if self.current_chunk == self.num_chunks:
+                self.current_chunk = 0
+
+            return allreduce_tensor
+
+        return None
 
 
 class MainParamsOptimizerWrapper(torch.optim.Optimizer):
@@ -109,7 +156,13 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
     """
 
     def __init__(
-        self, optimizer, fp32_grad_accum=False, contiguous_grad_bucket=False, async_grad_allreduce=False,
+        self,
+        optimizer,
+        fp32_grad_accum=False,
+        contiguous_grad_bucket=False,
+        async_grad_allreduce=False,
+        grad_div_ar_fusion=True,
+        grad_allreduce_chunk_size_mb=0,
     ):
         if not HAVE_APEX:
             raise ImportError(
@@ -131,8 +184,21 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
 
         self._fp32_grad_accum = fp32_grad_accum
         self._contiguous_grad_bucket = contiguous_grad_bucket
+
+        # used with tensor parallel only (no pipeline parallelism)
+        # be careful, weight update cannot start until all async grad AR works are done
         self._async_grad_allreduce = async_grad_allreduce
-        self._require_backward_grad_sync = False
+        self._grad_divisor = 1 / get_data_parallel_world_size()
+
+        if self._async_grad_allreduce:
+            # use @no_sync to disable backward grad sync during gradient accumulation
+            self._require_backward_grad_sync = True
+            self._grad_div_ar_fusion = grad_div_ar_fusion
+            self._grad_allreduce_chunk_size_mb = grad_allreduce_chunk_size_mb
+        else:
+            self._require_backward_grad_sync = False
+            self._grad_div_ar_fusion = False
+            self._grad_allreduce_chunk_size_mb = 0
 
         # Dummy tensor needed for apex multi-apply tensor.
         self._dummy_overflow_buf = None
@@ -151,7 +217,7 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
                         num_elements[i] = num_elements.get(i, 0) + param.data.nelement()
 
                 # Allocate gradient memory buffers for each data type
-                self._main_grad_buffers[i] = GradBucket(num_elements[i])
+                self._main_grad_buffers[i] = GradBucket(num_elements[i], self._grad_allreduce_chunk_size_mb)
 
         # Three groups of parameters:
         self.float16_groups = []  # original float16 parameters
@@ -185,7 +251,9 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
                         # Assign the grad buffer offset to main parameters
                         if self._contiguous_grad_bucket:
                             num_elements[i] -= param.data.nelement()
-                            main_param.grad = self._main_grad_buffers[i].get(param.data.shape, num_elements[i])
+                            main_param.grad, grad_chunk_info = self._main_grad_buffers[i].get(
+                                param.data.shape, num_elements[i]
+                            )
                             # Add a pointer to main_grad in model param for first-last stage embedding param reduction
                             param.main_grad = main_param.grad
 
@@ -215,7 +283,7 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
                     param_tmp = param.expand_as(param)
                     # Get the gradient accumulator function.
                     grad_acc = param_tmp.grad_fn.next_functions[0][0]
-                    grad_acc.register_hook(self._make_param_hook(param, main_param))
+                    grad_acc.register_hook(self._make_param_hook(param, main_param, i, grad_chunk_info))
                     self.grad_accs.append(grad_acc)
 
             self.float16_groups.append(float16_params_this_group)
@@ -226,12 +294,12 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
         # recast preexisting per-param state tensors
         self.optimizer.load_state_dict(self.optimizer.state_dict())
 
-    def _make_param_hook(self, param, main_param):
+    def _make_param_hook(self, param, main_param, i, grad_chunk_info):
         """Create the grad accumulation and all-reduce hook for backprop."""
         # Hook used for back-prop.
         def param_hook(*unused):
             # Accumulates gradients on main gradients
-            if param.grad.data is not None:
+            if param.grad is not None:
                 if main_param.grad is None:
                     main_param.grad = param.grad.float()
                 else:
@@ -239,10 +307,39 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
                 # Deallocate grad memory.
                 param.grad = None
 
-                # Asynchronous gradients allreduce accross data_parallel ranks
-                if self._require_backward_grad_sync:
-                    main_param.grad.div_(get_data_parallel_world_size())
-                    torch.distributed.all_reduce(main_param.grad, group=get_data_parallel_group(), async_op=True)
+            # Asynchronous gradients allreduce accross data_parallel ranks
+            if self._require_backward_grad_sync:
+                if self._grad_allreduce_chunk_size_mb > 0:
+                    self._main_grad_buffers[i].update_chunk_info(grad_chunk_info)
+                    while True:
+                        allreduce_tensor = self._main_grad_buffers[i].get_allreduce_tensor()
+                        if allreduce_tensor is None:
+                            break
+                        if self._grad_div_ar_fusion:
+                            torch.distributed.all_reduce(
+                                allreduce_tensor,
+                                group=get_data_parallel_group(),
+                                async_op=True,
+                                op=torch.distributed.make_nccl_premul_sum(self._grad_divisor),
+                            )
+                        else:
+                            allreduce_tensor.div_(get_data_parallel_world_size())
+                            torch.distributed.all_reduce(
+                                allreduce_tensor, group=get_data_parallel_group(), async_op=True,
+                            )
+                else:
+                    if self._grad_div_ar_fusion:
+                        torch.distributed.all_reduce(
+                            main_param.grad,
+                            group=get_data_parallel_group(),
+                            async_op=True,
+                            op=torch.distributed.make_nccl_premul_sum(self._grad_divisor),
+                        )
+                    else:
+                        main_param.grad.div_(get_data_parallel_world_size())
+                        torch.distributed.all_reduce(
+                            main_param.grad, group=get_data_parallel_group(), async_op=True,
+                        )
 
         return param_hook
 
@@ -311,6 +408,12 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
 
     @torch.no_grad()
     def step(self, **kwargs):
+        # while async grad allreduce is enabled, bprop will keep moving forward without waiting for
+        # the finish of async grad AR works. Hence, to guarantee the correctness of grads reduction,
+        # we cannot start weight update until all async grad AR works are done.
+        if self._async_grad_allreduce:
+            torch.cuda.synchronize()
+
         # Step the optimizer.
         self.optimizer.step(closure=None, **kwargs)
 
@@ -348,11 +451,11 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
             self._main_grad_buffers[i].allreduce_buffer()
 
     @contextmanager
-    def grad_sync(self):
+    def no_sync(self):
         """ A context manager to disable gradient synchronizations across
         data-parallel ranks."""
         old_require_backward_grad_sync = self._require_backward_grad_sync
-        self._require_backward_grad_sync = True
+        self._require_backward_grad_sync = False
         try:
             yield
         finally:
@@ -376,7 +479,10 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
     # Promote state so it can be retrieved or set via
     # "optimizer_instance.state"
     def _get_state(self):
-        return self.optimizer.state
+        if hasattr(self, 'optimizer'):
+            return self.optimizer.state
+        else:
+            return []
 
     def _set_state(self, value):
         self.optimizer.state = value
@@ -387,9 +493,25 @@ class MainParamsOptimizerWrapper(torch.optim.Optimizer):
     # "optimizer_instance.param_groups"
     # (for example, to adjust the learning rate)
     def _get_param_groups(self):
-        return self.optimizer.param_groups
+        if hasattr(self, 'optimizer'):
+            return self.optimizer.param_groups
+        else:
+            return []
 
     def _set_param_groups(self, value):
         self.optimizer.param_groups = value
 
     param_groups = property(_get_param_groups, _set_param_groups)
+
+    # Promote defaults so it can be retrieved or set via
+    # "optimizer_instance.defaults
+    def _get_defaults(self):
+        if hasattr(self, 'optimizer'):
+            return self.optimizer.defaults
+        else:
+            return []
+
+    def _set_defaults(self, value):
+        self.optimizer.defaults = value
+
+    defaults = property(_get_defaults, _set_defaults)
