@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import pynini
 from nemo_text_processing.inverse_text_normalization.en.utils import get_abs_path
 from nemo_text_processing.text_normalization.en.graph_utils import (
     NEMO_ALPHA,
@@ -21,22 +22,11 @@ from nemo_text_processing.text_normalization.en.graph_utils import (
     delete_extra_space,
     delete_space,
 )
+from pynini.lib import pynutil
 
-try:
-    import pynini
-    from pynini.lib import pynutil
-
-    graph_teen = pynini.string_file(get_abs_path("data/numbers/teen.tsv")).optimize()
-    graph_digit = pynini.string_file(get_abs_path("data/numbers/digit.tsv")).optimize()
-    ties_graph = pynini.string_file(get_abs_path("data/numbers/ties.tsv")).optimize()
-
-    PYNINI_AVAILABLE = True
-except (ModuleNotFoundError, ImportError):
-    graph_teen = None
-    graph_digit = None
-    ties_graph = None
-
-    PYNINI_AVAILABLE = True
+graph_teen = pynini.string_file(get_abs_path("data/numbers/teen.tsv")).optimize()
+graph_digit = pynini.string_file(get_abs_path("data/numbers/digit.tsv")).optimize()
+ties_graph = pynini.string_file(get_abs_path("data/numbers/ties.tsv")).optimize()
 
 
 def _get_month_graph():
