@@ -18,103 +18,60 @@ import string
 from pathlib import Path
 from typing import Dict
 
+import pynini
 from nemo_text_processing.text_normalization.en.utils import get_abs_path
+from pynini import Far
+from pynini.examples import plurals
+from pynini.export import export
+from pynini.lib import byte, pynutil, utf8
 
-try:
-    import pynini
-    from pynini import Far
-    from pynini.export import export
-    from pynini.examples import plurals
-    from pynini.lib import byte, pynutil, utf8
+NEMO_CHAR = utf8.VALID_UTF8_CHAR
 
-    NEMO_CHAR = utf8.VALID_UTF8_CHAR
+NEMO_DIGIT = byte.DIGIT
+NEMO_LOWER = pynini.union(*string.ascii_lowercase).optimize()
+NEMO_UPPER = pynini.union(*string.ascii_uppercase).optimize()
+NEMO_ALPHA = pynini.union(NEMO_LOWER, NEMO_UPPER).optimize()
+NEMO_ALNUM = pynini.union(NEMO_DIGIT, NEMO_ALPHA).optimize()
+NEMO_HEX = pynini.union(*string.hexdigits).optimize()
+NEMO_NON_BREAKING_SPACE = u"\u00A0"
+NEMO_SPACE = " "
+NEMO_WHITE_SPACE = pynini.union(" ", "\t", "\n", "\r", u"\u00A0").optimize()
+NEMO_NOT_SPACE = pynini.difference(NEMO_CHAR, NEMO_WHITE_SPACE).optimize()
+NEMO_NOT_QUOTE = pynini.difference(NEMO_CHAR, r'"').optimize()
 
-    NEMO_DIGIT = byte.DIGIT
-    NEMO_LOWER = pynini.union(*string.ascii_lowercase).optimize()
-    NEMO_UPPER = pynini.union(*string.ascii_uppercase).optimize()
-    NEMO_ALPHA = pynini.union(NEMO_LOWER, NEMO_UPPER).optimize()
-    NEMO_ALNUM = pynini.union(NEMO_DIGIT, NEMO_ALPHA).optimize()
-    NEMO_HEX = pynini.union(*string.hexdigits).optimize()
-    NEMO_NON_BREAKING_SPACE = u"\u00A0"
-    NEMO_SPACE = " "
-    NEMO_WHITE_SPACE = pynini.union(" ", "\t", "\n", "\r", u"\u00A0").optimize()
-    NEMO_NOT_SPACE = pynini.difference(NEMO_CHAR, NEMO_WHITE_SPACE).optimize()
-    NEMO_NOT_QUOTE = pynini.difference(NEMO_CHAR, r'"').optimize()
+NEMO_PUNCT = pynini.union(*map(pynini.escape, string.punctuation)).optimize()
+NEMO_GRAPH = pynini.union(NEMO_ALNUM, NEMO_PUNCT).optimize()
 
-    NEMO_PUNCT = pynini.union(*map(pynini.escape, string.punctuation)).optimize()
-    NEMO_GRAPH = pynini.union(NEMO_ALNUM, NEMO_PUNCT).optimize()
+NEMO_SIGMA = pynini.closure(NEMO_CHAR)
 
-    NEMO_SIGMA = pynini.closure(NEMO_CHAR)
+delete_space = pynutil.delete(pynini.closure(NEMO_WHITE_SPACE))
+delete_zero_or_one_space = pynutil.delete(pynini.closure(NEMO_WHITE_SPACE, 0, 1))
+insert_space = pynutil.insert(" ")
+delete_extra_space = pynini.cross(pynini.closure(NEMO_WHITE_SPACE, 1), " ")
+delete_preserve_order = pynini.closure(
+    pynutil.delete(" preserve_order: true")
+    | (pynutil.delete(" field_order: \"") + NEMO_NOT_QUOTE + pynutil.delete("\""))
+)
 
-    delete_space = pynutil.delete(pynini.closure(NEMO_WHITE_SPACE))
-    delete_zero_or_one_space = pynutil.delete(pynini.closure(NEMO_WHITE_SPACE, 0, 1))
-    insert_space = pynutil.insert(" ")
-    delete_extra_space = pynini.cross(pynini.closure(NEMO_WHITE_SPACE, 1), " ")
-    delete_preserve_order = pynini.closure(
-        pynutil.delete(" preserve_order: true")
-        | (pynutil.delete(" field_order: \"") + NEMO_NOT_QUOTE + pynutil.delete("\""))
-    )
+suppletive = pynini.string_file(get_abs_path("data/suppletive.tsv"))
+# _v = pynini.union("a", "e", "i", "o", "u")
+_c = pynini.union(
+    "b", "c", "d", "f", "g", "h", "j", "k", "l", "m", "n", "p", "q", "r", "s", "t", "v", "w", "x", "y", "z"
+)
+_ies = NEMO_SIGMA + _c + pynini.cross("y", "ies")
+_es = NEMO_SIGMA + pynini.union("s", "sh", "ch", "x", "z") + pynutil.insert("es")
+_s = NEMO_SIGMA + pynutil.insert("s")
 
-    suppletive = pynini.string_file(get_abs_path("data/suppletive.tsv"))
-    # _v = pynini.union("a", "e", "i", "o", "u")
-    _c = pynini.union(
-        "b", "c", "d", "f", "g", "h", "j", "k", "l", "m", "n", "p", "q", "r", "s", "t", "v", "w", "x", "y", "z"
-    )
-    _ies = NEMO_SIGMA + _c + pynini.cross("y", "ies")
-    _es = NEMO_SIGMA + pynini.union("s", "sh", "ch", "x", "z") + pynutil.insert("es")
-    _s = NEMO_SIGMA + pynutil.insert("s")
+graph_plural = plurals._priority_union(
+    suppletive, plurals._priority_union(_ies, plurals._priority_union(_es, _s, NEMO_SIGMA), NEMO_SIGMA), NEMO_SIGMA
+).optimize()
 
-    graph_plural = plurals._priority_union(
-        suppletive, plurals._priority_union(_ies, plurals._priority_union(_es, _s, NEMO_SIGMA), NEMO_SIGMA), NEMO_SIGMA
-    ).optimize()
-
-    SINGULAR_TO_PLURAL = graph_plural
-    PLURAL_TO_SINGULAR = pynini.invert(graph_plural)
-    TO_LOWER = pynini.union(*[pynini.cross(x, y) for x, y in zip(string.ascii_uppercase, string.ascii_lowercase)])
-    TO_UPPER = pynini.invert(TO_LOWER)
-
-    PYNINI_AVAILABLE = True
-except (ModuleNotFoundError, ImportError):
-    # Create placeholders
-    NEMO_CHAR = None
-
-    NEMO_DIGIT = None
-    NEMO_LOWER = None
-    NEMO_UPPER = None
-    NEMO_ALPHA = None
-    NEMO_ALNUM = None
-    NEMO_HEX = None
-    NEMO_NON_BREAKING_SPACE = u"\u00A0"
-    NEMO_SPACE = " "
-    NEMO_WHITE_SPACE = None
-    NEMO_NOT_SPACE = None
-    NEMO_NOT_QUOTE = None
-
-    NEMO_PUNCT = None
-    NEMO_GRAPH = None
-
-    NEMO_SIGMA = None
-
-    delete_space = None
-    insert_space = None
-    delete_extra_space = None
-    delete_preserve_order = None
-
-    suppletive = None
-    # _v = pynini.union("a", "e", "i", "o", "u")
-    _c = None
-    _ies = None
-    _es = None
-    _s = None
-
-    graph_plural = None
-
-    SINGULAR_TO_PLURAL = None
-    PLURAL_TO_SINGULAR = None
-    TO_LOWER = None
-    TO_UPPER = None
-
-    PYNINI_AVAILABLE = False
+SINGULAR_TO_PLURAL = graph_plural
+PLURAL_TO_SINGULAR = pynini.invert(graph_plural)
+TO_LOWER = pynini.union(*[pynini.cross(x, y) for x, y in zip(string.ascii_uppercase, string.ascii_lowercase)])
+TO_UPPER = pynini.invert(TO_LOWER)
+MIN_NEG_WEIGHT = -0.0001
+MIN_POS_WEIGHT = 0.0001
 
 
 def generator_main(file_name: str, graphs: Dict[str, 'pynini.FstLike']):
