@@ -12,20 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import pynini
 from nemo_text_processing.text_normalization.en.graph_utils import NEMO_SIGMA, TO_UPPER, GraphFst, get_abs_path
+from pynini.lib import pynutil
 
-try:
-    import pynini
-    from pynini.lib import pynutil
-
-    delete_space = pynutil.delete(" ")
-    quantities = pynini.string_file(get_abs_path("data/number/thousand.tsv"))
-    quantities_abbr = pynini.string_file(get_abs_path("data/number/quantity_abbr.tsv"))
-    quantities_abbr |= TO_UPPER @ quantities_abbr
-
-    PYNINI_AVAILABLE = True
-except (ModuleNotFoundError, ImportError):
-    PYNINI_AVAILABLE = False
+delete_space = pynutil.delete(" ")
+quantities = pynini.string_file(get_abs_path("data/number/thousand.tsv"))
+quantities_abbr = pynini.string_file(get_abs_path("data/number/quantity_abbr.tsv"))
+quantities_abbr |= TO_UPPER @ quantities_abbr
 
 
 def get_quantity(
@@ -48,7 +42,7 @@ def get_quantity(
         + cardinal_up_to_hundred
         + pynutil.insert("\"")
         + pynini.closure(pynutil.delete(" "), 0, 1)
-        + pynutil.insert("quantity: \"")
+        + pynutil.insert(" quantity: \"")
         + (quantity_wo_thousand @ (quantities | quantities_abbr))
         + pynutil.insert("\"")
     )
