@@ -13,12 +13,18 @@
 # limitations under the License.
 
 import pytest
-from nemo_text_processing.inverse_text_normalization.inverse_normalize import InverseNormalizer
-from nemo_text_processing.text_normalization.normalize import Normalizer
-from nemo_text_processing.text_normalization.normalize_with_audio import NormalizerWithAudio
 from parameterized import parameterized
 
-from ..utils import CACHE_DIR, PYNINI_AVAILABLE, parse_test_case_file
+from ..utils import CACHE_DIR, RUN_AUDIO_BASED_TESTS, parse_test_case_file
+
+try:
+    from nemo_text_processing.inverse_text_normalization.inverse_normalize import InverseNormalizer
+    from nemo_text_processing.text_normalization.normalize import Normalizer
+    from nemo_text_processing.text_normalization.normalize_with_audio import NormalizerWithAudio
+
+    PYNINI_AVAILABLE = True
+except (ImportError, ModuleNotFoundError):
+    PYNINI_AVAILABLE = False
 
 
 class TestCardinal:
@@ -29,7 +35,8 @@ class TestCardinal:
 
     @parameterized.expand(parse_test_case_file('es/data_inverse_text_normalization/test_cases_cardinal.txt'))
     @pytest.mark.skipif(
-        not PYNINI_AVAILABLE, reason="`pynini` not installed, please install via nemo_text_processing/setup.sh"
+        not PYNINI_AVAILABLE,
+        reason="`pynini` not installed, please install via nemo_text_processing/pynini_install.sh",
     )
     @pytest.mark.run_only_on('CPU')
     @pytest.mark.unit
@@ -45,13 +52,16 @@ class TestCardinal:
 
     normalizer_with_audio = (
         NormalizerWithAudio(input_case='cased', lang='es', cache_dir=CACHE_DIR, overwrite_cache=False)
-        if PYNINI_AVAILABLE and CACHE_DIR
+        if PYNINI_AVAILABLE
+        else None
+        if CACHE_DIR and RUN_AUDIO_BASED_TESTS and PYNINI_AVAILABLE
         else None
     )
 
     @parameterized.expand(parse_test_case_file('es/data_text_normalization/test_cases_cardinal.txt'))
     @pytest.mark.skipif(
-        not PYNINI_AVAILABLE, reason="`pynini` not installed, please install via nemo_text_processing/setup.sh"
+        not PYNINI_AVAILABLE,
+        reason="`pynini` not installed, please install via nemo_text_processing/pynini_install.sh",
     )
     @pytest.mark.run_only_on('CPU')
     @pytest.mark.unit
